# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/',
                'doc_host': 'https://claudette.answer.ai',
                'git_url': 'https://github.com/AnswerDotAI/claudette',
                'lib_path': 'claudette'},
  'syms': { 'claudette.asink': { 'claudette.asink.AsyncChat': ('async.html#asyncchat', 'claudette/asink.py'),
                                 'claudette.asink.AsyncChat.__call__': ('async.html#asyncchat.__call__', 'claudette/asink.py'),
                                 'claudette.asink.AsyncChat.__init__': ('async.html#asyncchat.__init__', 'claudette/asink.py'),
                                 'claudette.asink.AsyncChat._append_pr': ('async.html#asyncchat._append_pr', 'claudette/asink.py'),
                                 'claudette.asink.AsyncClient': ('async.html#asyncclient', 'claudette/asink.py'),
                                 'claudette.asink.AsyncClient.__call__': ('async.html#asyncclient.__call__', 'claudette/asink.py'),
                                 'claudette.asink.AsyncClient.__init__': ('async.html#asyncclient.__init__', 'claudette/asink.py'),
                                 'claudette.asink.AsyncClient.structured': ('async.html#asyncclient.structured', 'claudette/asink.py'),
                                 'claudette.asink._astream': ('async.html#_astream', 'claudette/asink.py'),
                                 'claudette.asink.mk_funcres_async': ('async.html#mk_funcres_async', 'claudette/asink.py'),
                                 'claudette.asink.mk_toolres_async': ('async.html#mk_toolres_async', 'claudette/asink.py')},
            'claudette.core': { 'claudette.core.Chat': ('core.html#chat', 'claudette/core.py'),
                                'claudette.core.Chat.__call__': ('core.html#chat.__call__', 'claudette/core.py'),
                                'claudette.core.Chat.__init__': ('core.html#chat.__init__', 'claudette/core.py'),
                                'claudette.core.Chat._append_pr': ('core.html#chat._append_pr', 'claudette/core.py'),
                                'claudette.core.Chat._post_pr': ('core.html#chat._post_pr', 'claudette/core.py'),
                                'claudette.core.Chat._repr_markdown_': ('core.html#chat._repr_markdown_', 'claudette/core.py'),
                                'claudette.core.Chat.cost': ('core.html#chat.cost', 'claudette/core.py'),
                                'claudette.core.Chat.use': ('core.html#chat.use', 'claudette/core.py'),
                                'claudette.core.Client': ('core.html#client', 'claudette/core.py'),
                                'claudette.core.Client.__call__': ('core.html#client.__call__', 'claudette/core.py'),
                                'claudette.core.Client.__init__': ('core.html#client.__init__', 'claudette/core.py'),
                                'claudette.core.Client._log': ('core.html#client._log', 'claudette/core.py'),
                                'claudette.core.Client._precall': ('core.html#client._precall', 'claudette/core.py'),
                                'claudette.core.Client._r': ('core.html#client._r', 'claudette/core.py'),
                                'claudette.core.Client._repr_markdown_': ('core.html#client._repr_markdown_', 'claudette/core.py'),
                                'claudette.core.Client.cost': ('core.html#client.cost', 'claudette/core.py'),
                                'claudette.core.Client.structured': ('core.html#client.structured', 'claudette/core.py'),
                                'claudette.core.Message._repr_markdown_': ('core.html#message._repr_markdown_', 'claudette/core.py'),
                                'claudette.core.ServerToolUsage.__add__': ('core.html#servertoolusage.__add__', 'claudette/core.py'),
                                'claudette.core.ToolResult': ('core.html#toolresult', 'claudette/core.py'),
                                'claudette.core.ToolResult.__init__': ('core.html#toolresult.__init__', 'claudette/core.py'),
                                'claudette.core.ToolResult.__str__': ('core.html#toolresult.__str__', 'claudette/core.py'),
                                'claudette.core.Usage.__add__': ('core.html#usage.__add__', 'claudette/core.py'),
                                'claudette.core.Usage.__repr__': ('core.html#usage.__repr__', 'claudette/core.py'),
                                'claudette.core.Usage.cost': ('core.html#usage.cost', 'claudette/core.py'),
                                'claudette.core.Usage.total': ('core.html#usage.total', 'claudette/core.py'),
                                'claudette.core._convert': ('core.html#_convert', 'claudette/core.py'),
                                'claudette.core._dgetattr': ('core.html#_dgetattr', 'claudette/core.py'),
                                'claudette.core._img_content': ('core.html#_img_content', 'claudette/core.py'),
                                'claudette.core._is_builtin': ('core.html#_is_builtin', 'claudette/core.py'),
                                'claudette.core._stream': ('core.html#_stream', 'claudette/core.py'),
                                'claudette.core._type': ('core.html#_type', 'claudette/core.py'),
                                'claudette.core.blks2cited_txt': ('core.html#blks2cited_txt', 'claudette/core.py'),
                                'claudette.core.can_set_system_prompt': ('core.html#can_set_system_prompt', 'claudette/core.py'),
                                'claudette.core.can_set_temperature': ('core.html#can_set_temperature', 'claudette/core.py'),
                                'claudette.core.can_stream': ('core.html#can_stream', 'claudette/core.py'),
                                'claudette.core.can_use_extended_thinking': ('core.html#can_use_extended_thinking', 'claudette/core.py'),
                                'claudette.core.contents': ('core.html#contents', 'claudette/core.py'),
                                'claudette.core.find_block': ('core.html#find_block', 'claudette/core.py'),
                                'claudette.core.find_blocks': ('core.html#find_blocks', 'claudette/core.py'),
                                'claudette.core.get_costs': ('core.html#get_costs', 'claudette/core.py'),
                                'claudette.core.get_pricing': ('core.html#get_pricing', 'claudette/core.py'),
                                'claudette.core.get_types': ('core.html#get_types', 'claudette/core.py'),
                                'claudette.core.mk_funcres': ('core.html#mk_funcres', 'claudette/core.py'),
                                'claudette.core.mk_tool_choice': ('core.html#mk_tool_choice', 'claudette/core.py'),
                                'claudette.core.mk_toolres': ('core.html#mk_toolres', 'claudette/core.py'),
                                'claudette.core.search_conf': ('core.html#search_conf', 'claudette/core.py'),
                                'claudette.core.server_tool_usage': ('core.html#server_tool_usage', 'claudette/core.py'),
                                'claudette.core.think_md': ('core.html#think_md', 'claudette/core.py'),
                                'claudette.core.tool': ('core.html#tool', 'claudette/core.py'),
                                'claudette.core.usage': ('core.html#usage', 'claudette/core.py')},
            'claudette.text_editor': { 'claudette.text_editor.create': ('text_editor.html#create', 'claudette/text_editor.py'),
                                       'claudette.text_editor.insert': ('text_editor.html#insert', 'claudette/text_editor.py'),
                                       'claudette.text_editor.str_replace': ('text_editor.html#str_replace', 'claudette/text_editor.py'),
                                       'claudette.text_editor.str_replace_based_edit_tool': ( 'text_editor.html#str_replace_based_edit_tool',
                                                                                              'claudette/text_editor.py'),
                                       'claudette.text_editor.str_replace_editor': ( 'text_editor.html#str_replace_editor',
                                                                                     'claudette/text_editor.py'),
                                       'claudette.text_editor.view': ('text_editor.html#view', 'claudette/text_editor.py')},
            'claudette.toolloop': { 'claudette.toolloop.AsyncChat.toolloop': ('toolloop.html#asyncchat.toolloop', 'claudette/toolloop.py'),
                                    'claudette.toolloop.Chat.toolloop': ('toolloop.html#chat.toolloop', 'claudette/toolloop.py')}}}
